/***************************************************************************
 *                  GSkyRegions.hpp - Sky region container class           *
 * ----------------------------------------------------------------------- *
 *  copyright (C) 2009-2013 by Juergen Knoedlseder                         *
 * ----------------------------------------------------------------------- *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 *                                                                         *
 ***************************************************************************/
/**
 * @file GSkyRegions.hpp
 * @brief Sky regions container class definition
 * @author Pierrick martin
 */

#ifndef GSKYREGIONS_HPP
#define GSKYREGIONS_HPP

/* __ Includes ___________________________________________________________ */
#include <string>
#include <vector>
#include "GBase.hpp"
#include "GXml.hpp"
#include "GSkyRegion.hpp"


/***********************************************************************//**
* @class GSkyRegions
*
* @brief Sky region container class
*
* This container class collects sky regions of type GSkyRegion that are 
* used to extract properties over regions of the sky (events, exposure,...)
* The names of all regions in the container have to be unique, i.e. every 
* name can occur only once. This allows for accessing the regions by name 
* and by index.
*
* To be clarified:
* - Role/need of at() and reserve() methods ?
* - Role/need of load/save and read/write methods (especially if DS9 file format) ?
 ***************************************************************************/
class GSkyRegions : public GBase {

public:
    // Constructors and destructors
    GSkyRegions(void);
    GSkyRegions(const GSkyRegions& regions);;
    virtual ~GSkyRegions(void);

    // Operators
    GSkyRegions&      operator=(const GSkyRegions& regions);
    GSkyRegion*       operator[](const int& index);
    const GSkyRegion* operator[](const int& index) const;
    GSkyRegion*       operator[](const std::string& name);
    const GSkyRegion* operator[](const std::string& name) const;

    // Methods
    void              clear(void);
    GSkyRegions*      clone(void) const;
    GSkyRegion*       at(const int& index);
    const GSkyRegion* at(const int& index) const;
    int               size(void) const;
    bool              isempty(void) const;
    GSkyRegion*       set(const int& index, const GSkyRegion& region);
    GSkyRegion*       set(const std::string& name, const GSkyRegion& region);
    GSkyRegion*       append(const GSkyRegion& region);
    GSkyRegion*       insert(const int& index, const GSkyRegion& region);
    GSkyRegion*       insert(const std::string& name, const GSkyRegion& region);
    void              remove(const int& index);
    void              remove(const std::string& name);
    void              reserve(const int& num);
    void              extend(const GSkyRegions& regions);
    bool              contains(const std::string& name) const;
    void              load(const std::string& filename);
    void              save(const std::string& filename) const;
    void              read(const GXml& xml);
    void              write(GXml& xml) const;
    std::string       print(const GChatter& chatter = NORMAL) const;

protected:
    // Protected methods
    void          init_members(void);
    void          copy_members(const GSkyRegions& regions);
    void          free_members(void);
    void          set_pointers(void);
    int           get_index(const std::string& name) const;

    // Protected members
    std::vector<GSkyRegion*> m_regions;  //!< List of regions
};


/***********************************************************************//**
 * @brief Return pointer to region
 *
 * @param[in] index region index [0,...,size()-1].
 *
 * Returns a pointer to the region with the specified @p index.
 ***************************************************************************/
inline
GSkyRegion* GSkyRegions::operator[](const int& index)
{
    return (m_regions[index]);
}


/***********************************************************************//**
 * @brief Return pointer to region (const version)
 *
 * @param[in] index region index [0,...,size()-1].
 *
 * Returns a const pointer to the region with the specified @p index.
 ***************************************************************************/
inline
const GSkyRegion* GSkyRegions::operator[](const int& index) const
{
    return (m_regions[index]);
}


/***********************************************************************//**
 * @brief Return number of regions in container
 *
 * @return Number of regions in container.
 *
 * Returns the number of regions in the region container.
 ***************************************************************************/
inline
int GSkyRegions::size(void) const
{
    return (m_regions.size());
}


/***********************************************************************//**
 * @brief Signals if there are no regions in container
 *
 * @return True if container is empty, false otherwise.
 *
 * Signals if the region container does not contain any region.
 ***************************************************************************/
inline
bool GSkyRegions::isempty(void) const
{
    return (m_regions.empty());
}


/***********************************************************************//**
 * @brief Reserves space for regions in container
 *
 * @param[in] num Number of regions
 *
 * Reserves space for @p num regions in the container.
 ***************************************************************************/
inline
void GSkyRegions::reserve(const int& num)
{
    m_regions.reserve(num);
    return;
}

#endif /* GSKYREGIONS_HPP */